
local Store = {}

local function checkCCStore()
    if not CCStore then
        echoError("framework.api.Store - CCStore not exists.")
        return false
    end
    return true
end

function Store.init(listener)
    if not checkCCStore() then return false end

    if cct.storeProvider then
        echoError("Store.init() - store already init")
        return false
    end

    if type(listener) ~= "function" then
        echoError("Store.init() - invalid listener")
        return false
    end

    cct.storeProvider = CCStore:sharedStore() -- avoid gc
    return cct.storeProvider:postInitWithTransactionListener(listener)
end

function Store.getReceiptVerifyMode()
    if not checkCCStore() then return false end
    return cct.storeProvider:getReceiptVerifyMode()
end

function Store.setReceiptVerifyMode(mode, isSandbox)
    if not checkCCStore() then return false end

    if type(mode) ~= "number"
        or (mode ~= CCStoreReceiptVerifyModeNone
            and mode ~= CCStoreReceiptVerifyModeDevice
            and mode ~= CCStoreReceiptVerifyModeServer) then
        echoError("Store.setReceiptVerifyMode() - invalid mode")
        return false
    end

    if type(isSandbox) ~= "boolean" then isSandbox = true end
    cct.storeProvider:setReceiptVerifyMode(mode, isSandbox)
end

function Store.getReceiptVerifyServerUrl()
    if not checkCCStore() then return false end
    return cct.storeProvider:getReceiptVerifyServerUrl()
end

function Store.setReceiptVerifyServerUrl(url)
    if not checkCCStore() then return false end

    if type(url) ~= "string" then
        echoError("Store.setReceiptVerifyServerUrl() - invalid url")
        return false
    end
    cct.storeProvider:setReceiptVerifyServerUrl(url)
end

function Store.canMakePurchases()
    if not checkCCStore() then return false end
    return cct.storeProvider:canMakePurchases()
end

function Store.loadProducts(productsId, listener)
    if not checkCCStore() then return false end

    if type(listener) ~= "function" then
        echoError("Store.loadProducts() - invalid listener")
        return false
    end

    if type(productsId) ~= "table" then
        echoError("Store.loadProducts() - invalid productsId")
        return false
    end

    for i = 1, #productsId do
        if type(productsId[i]) ~= "string" then
            echoError("Store.loadProducts() - invalid id[#%d] in productsId", i)
            return false
        end
    end

    cct.storeProvider:loadProducts(productsId, listener)
    return true
end

function Store.cancelLoadProducts()
    if not checkCCStore() then return false end
    cct.storeProvider:cancelLoadProducts()
end

function Store.isProductLoaded(productId)
    if not checkCCStore() then return false end
    return cct.storeProvider:isProductLoaded(productId)
end

function Store.purchase(productId)
    if not checkCCStore() then return false end

    if not cct.storeProvider then
        echoError("Store.purchase() - store not init")
        return false
    end

    if type(productId) ~= "string" then
        echoError("Store.purchase() - invalid productId")
        return false
    end

    return cct.storeProvider:purchase(productId)
end

function Store.restore()
    if not checkCCStore() then return false end
    cct.storeProvider:restore()
end

function Store.finishTransaction(transaction)
    if not checkCCStore() then return false end

    if not cct.storeProvider then
        echoError("Store.finishTransaction() - store not init")
        return false
    end

    if type(transaction) ~= "table" or type(transaction.transactionIdentifier) ~= "string" then
        echoError("Store.finishTransaction() - invalid transaction")
        return false
    end

    return cct.storeProvider:finishTransaction(transaction.transactionIdentifier)
end

return Store
