
CCNodeExtend = class("CCNodeExtend")
CCNodeExtend.__index = CCNodeExtend

function CCNodeExtend.extend(target)
    local t = tolua.getpeer(target)
    if not t then
        t = {}
        tolua.setpeer(target, t)
    end
    setmetatable(t, CCNodeExtend)
    return target
end

function CCNodeExtend:align(anchorPoint, x, y)
    self:setAnchorPoint(display.ANCHOR_POINTS[anchorPoint])
    if x and y then self:setPosition(x, y) end
    return self
end

function CCNodeExtend:schedule(callback, interval)
    local seq = transition.sequence({
        CCDelayTime:create(interval),
        CCCallFunc:create(callback),
    })
    local action = CCRepeatForever:create(seq)
    self:runAction(action)
    return action
end

function CCNodeExtend:performWithDelay(callback, delay)
    local action = transition.sequence({
        CCDelayTime:create(delay),
        CCCallFunc:create(callback),
    })
    self:runAction(action)
    return action
end

function CCNodeExtend:onEnter()
end

function CCNodeExtend:onExit()
end

function CCNodeExtend:onEnterTransitionFinish()
end

function CCNodeExtend:onExitTransitionStart()
end

function CCNodeExtend:onCleanup()
end

function CCNodeExtend:setNodeEventEnabled(enabled, handler)
    if enabled then
        if not handler then
            handler = function(event)
                if event == "enter" then
                    self:onEnter()
                elseif event == "exit" then
                    self:onExit()
                elseif event == "enterTransitionFinish" then
                    self:onEnterTransitionFinish()
                elseif event == "exitTransitionStart" then
                    self:onExitTransitionStart()
                elseif event == "cleanup" then
                    self:onCleanup()
                end
            end
        end
        self:registerScriptHandler(handler)
    else
        self:unregisterScriptHandler()
    end
    return self
end

function CCNodeExtend:removeSelf(cleanup)
    if not tolua.isnull(self) then
        if cleanup ~= false then cleanup = true end
        self:removeFromParentAndCleanup(cleanup)
    end
end

local function _getPosInGobal( node )
    return node:getParent():convertToWorldSpace(
            cc.p(node:getPositionX(), node:getPositionY()))
end

function CCNodeExtend:_setNowSelectIdxByGivenPos(nextPos)
    for i = 1, #self.imageMenuItems_ do
        local pos = _getPosInGobal( self.imageMenuItems_[i] )
        if pos.x == nextPos.x and pos.y == nextPos.y then
            self.imageMenuItems_nowSelectIdx = i
            break
        end
    end
end

function CCNodeExtend:isAllImageMenuEnable(b)
    self.imageMenuItems_isEnable = b
    if self.imageMenuItems_isEnable then
        -- self:_cursorMoveTo("down")
    else
        if type(self.imageMenuItems_) == "table" then
            for i = 1, #self.imageMenuItems_ do
                if self.imageMenuItems_selectedHalo[i] ~= nil then
                    self.imageMenuItems_selectedHalo[i]:setVisible(false)
                end
            end
        end
    end
    app:notifiIsJoystickMode(not b)
end

function CCNodeExtend:_setSelectedImageMenuItem()
    for i = 1, #self.imageMenuItems_ do
        if i == self.imageMenuItems_nowSelectIdx then
            self.imageMenuItems_selectedHalo[i]:setVisible(true)
        else
            self.imageMenuItems_selectedHalo[i]:setVisible(false)
        end
    end
end

function CCNodeExtend:_getClickablePoss()
    local aPos = {}
    for i = 1, #self.imageMenuItems_ do
        if (self.imageMenuItems_[i]:isEnabled() and self.imageMenuItems_[i]:isVisible()) then
            aPos[#aPos + 1] = _getPosInGobal( self.imageMenuItems_[i] )
        end
    end
    if #aPos == 0 then
        assert(false, "must have enabled btn!")
    end
    return aPos
end

local function _sortAPosFromUpToDown( a, b )
    if a.y == b.y then
        return a.x < b.x
    else
        return a.y > b.y
    end
end

local function _sortAPosFromLeftToRight( a, b )
    if a.x == b.x then
        return a.y > b.y
    else
        return a.x < b.x
    end
end

local function _findNextPosByGivenDirection( dir, allPosTab, curPos)

    local function judgeIsEdgeAndReturnNextPos( pos )
        local edgePos = nil
        local nextPos = nil
        if dir == "up" or dir == "left" then
            edgePos = allPosTab[1]
            nextPos = allPosTab[#allPosTab]
        elseif dir == "down" or dir == "right" then
            edgePos = allPosTab[#allPosTab]
            nextPos = allPosTab[1]
        end
        if pos.x == edgePos.x and pos.y == edgePos.y then
            return true, nextPos
        else
            return false
        end
    end
    local isEdge, eNextPos = judgeIsEdgeAndReturnNextPos( curPos )
    if isEdge then
        return eNextPos
    else
        local nextAvailablePoss = {}

        for i = 1, #allPosTab do
            local function isAvailabe()
                if dir == "up" then
                    return ((allPosTab[i].y > curPos.y) or (allPosTab[i].y == curPos.y and allPosTab[i].x < curPos.x))
                elseif dir == "down" then
                    return ((allPosTab[i].y < curPos.y) or (allPosTab[i].y == curPos.y and allPosTab[i].x > curPos.x))
                elseif dir == "left" then
                    return ((allPosTab[i].x < curPos.x) or (allPosTab[i].x == curPos.x and allPosTab[i].y < curPos.y))
                elseif dir == "right" then
                    return ((allPosTab[i].x > curPos.x) or (allPosTab[i].x == curPos.x and allPosTab[i].y > curPos.y))
                end
            end
            if isAvailabe() then
                nextAvailablePoss[#nextAvailablePoss + 1] = allPosTab[i]
            end
        end


        if #nextAvailablePoss ~= 0 then
            local function sortFun( a, b )
                if cc.pGetDistance(a, curPos) < cc.pGetDistance(b, curPos) then
                    return true
                else
                    return false
                end
            end
            table.sort(nextAvailablePoss, sortFun)
            return nextAvailablePoss[1]
        else
            assert(false, "next position must have one")
        end
    end
end

function CCNodeExtend:_cursorMoveTo(dir)
    if #self.imageMenuItems_ > 0 and self.imageMenuItems_isEnable then
        local aPos = self:_getClickablePoss()

        local sortFunName = _sortAPosFromUpToDown
        if dir == "left" or dir == "right" then
            sortFunName = _sortAPosFromLeftToRight
        end

        table.sort(aPos, sortFunName)

        local curPos = _getPosInGobal(self.imageMenuItems_[self.imageMenuItems_nowSelectIdx])
        local nextPos = _findNextPosByGivenDirection( dir, aPos, curPos)

        self:_setNowSelectIdxByGivenPos(nextPos)

        self:_setSelectedImageMenuItem()
    end
end

function CCNodeExtend:_setDefualSelectItem()
    if self.imageMenuItems_isEnable then
        local aPos = self:_getClickablePoss()
        local selectedPos = aPos[1]

        local defualPos = _getPosInGobal(self.imageMenuItems_defualSelectItems[#self.imageMenuItems_defualSelectItems])

        for i = 1, #aPos do
            if aPos[i].x == defualPos.x and aPos[i].y == defualPos.y then
                selectedPos = defualPos
                break
            end
        end

        self:_setNowSelectIdxByGivenPos(selectedPos)

        self:_setSelectedImageMenuItem()
    else
        -- print("skip!!")
    end
end

-- ouput api
-- move cursor direction
function CCNodeExtend:cursorMoveTo( dir )
    if dir == "up" or dir == "down" or dir == "left" or dir == "right" then
        self:_cursorMoveTo(dir)
    else
        assert(false, "wrong direction!!")
    end
end
-- click current imageMenuItem
function CCNodeExtend:clickCurrentImageMenuItem()
    if #self.imageMenuItems_ > 0 and self.imageMenuItems_isEnable and self.imageMenuItems_[self.imageMenuItems_nowSelectIdx]:isEnabled() then
        self.imageMenuItems_[self.imageMenuItems_nowSelectIdx]:activate()
    end
end

function CCNodeExtend:setDefualSelectItem( item )

    self.imageMenuItems_nowSelectIdx = 1

    local isAdd = true
    for i = 1, #self.imageMenuItems_defualSelectItems do
        if self.imageMenuItems_defualSelectItems[i] == item then
            isAdd = false
            break
        end
    end

    if isAdd then
        self.imageMenuItems_defualSelectItems[#self.imageMenuItems_defualSelectItems + 1] = item
    end

    local handle
    handle = CCDirector:getInstance():getScheduler():scheduleScriptFunc(function()
        runningScene = cc.Director:getInstance():getRunningScene()
        if tostring(tolua.type(runningScene)) == "cc.Scene" then

            self:_setDefualSelectItem()

            CCDirector:getInstance():getScheduler():unscheduleScriptEntry(handle)
        end
    end, 0, false)
end

function CCNodeExtend:delMaySelectItem(item)
    local atTableIdx = -1
    for i =1, #self.imageMenuItems_ do
        if self.imageMenuItems_[i] == item then
            atTableIdx = i
            break
        end
    end

    if atTableIdx ~= -1 then
        self.imageMenuItems_selectedHalo[atTableIdx]:setVisible(false)
        self.imageMenuItems_selectedHalo[atTableIdx] = nil
        table.remove(self.imageMenuItems_selectedHalo, atTableIdx)

        self.imageMenuItems_[atTableIdx]:setVisible(false)
        self.imageMenuItems_[atTableIdx] = nil
        table.remove(self.imageMenuItems_, atTableIdx)
    else
        print("fail del may select item")
    end

    local defualIdx = -1
    for i = 1, #self.imageMenuItems_defualSelectItems do
        if self.imageMenuItems_defualSelectItems[i] == item then
            defualIdx = i
            break
        end
    end

    if defualIdx ~= -1 then
        local handle
        handle = CCDirector:getInstance():getScheduler():scheduleScriptFunc(function()
            for i = 1, #self.imageMenuItems_ do
                if (self.imageMenuItems_[i]:isEnabled() and self.imageMenuItems_[i]:isVisible()) then
                    self.imageMenuItems_defualSelectItems[defualIdx] = nil
                    table.remove(self.imageMenuItems_defualSelectItems, defualIdx)
                    self:_setDefualSelectItem()
                    CCDirector:getInstance():getScheduler():unscheduleScriptEntry(handle)
                    break
                end
            end
        end, 0, false)
    else
        print("fail set defual select item")
    end
end

function CCNodeExtend:addMaySelectItem(item)
    if self.imageMenuItems_ == nil then
        self.imageMenuItems_nowSelectIdx = 1
        self.imageMenuItems_isEnable = true
        self.imageMenuItems_defualSelectItems = {}

        self.imageMenuItems_ = {}
    end
    self.imageMenuItems_[#self.imageMenuItems_ + 1] = item

    if self.imageMenuItems_selectedHalo == nil then
        self.imageMenuItems_selectedHalo = {}
    end
    local halo = display.newScale9Sprite("bg/select_bg.png"
                , item:getContentSize().width/2
                , item:getContentSize().height/2
                , cc.size(item:getContentSize().width
                    , item:getContentSize().height)
                )
    self.imageMenuItems_selectedHalo[#self.imageMenuItems_selectedHalo + 1] = halo
    item:addChild(halo)
    halo:setVisible(false)
end
